
module Bender
  class ServiceImplementation

    attr_reader :name, :driver_name, :host_identity
    attr_reader :config, :base_config, :svc_config, :templates_path, :config_path
    attr_reader :plugins_path, :native_plugins_path, :plugin_list
    attr_reader :log, :backend

    def initialize(name, host_identity, config, backend)
      @name = name
      @config, @backend, @log = config, backend, backend.log
      @driver_name = self.class.name.demodulize.underscore
      @host_identity = host_identity
      @base_config = ::Tomte.config.bender[:base_config].recursively_symbolize_keys!

      @templates_path = base_config[@templates_path] || gem_templates_path
      @config_root   = base_config[@config_root]

      @svc_config = base_config[driver_name.to_sym]

      initialize_plugins

      log.warn "- doing a dry run".important! if dry_run?
      raise ArgumentError, "- Missing base configuration for service '#{driver_name}'" unless svc_config
    end

    def tasks
      puts "service #{name} has no tasks".yellow
    end

    def service_persistent_store
      unless backend.store.get(name.to_s)
        backend.store.set name.to_s, Marshal::dump({})
      end
      Marshal::load(backend.store.get(name.to_s))
    end

    def service_persistent_store=(value)
      backend.store.set name.to_s, Marshal::dump(value)
    end

    def last_update
      ts = service_persistent_store['last_updated'] || "2000-01-01"
      ts = DateTime.parse(ts) if ts.is_a? String
      ts
    end

    def last_update=(date)
      self.service_persistent_store = service_persistent_store.merge({'last_updated' => date })
    end


    def initialize_plugins
      @plugins_path = File.join("#{::Tomte.config.core[:application_lib]}/bender/services/", driver_name, 'plugins')
      @native_plugins_path = File.join(plugins_path, 'native')

      return unless Dir.exists?(plugins_path)
      list = svc_config[:plugins]

      @plugin_list = []

      list.each do | plugin |

        include_file = File.join(plugins_path, plugin[:file])
        next unless File.file?(include_file)

        log.debug "+ Plugins: including plugin '#{plugin[:name]}'"
        require(include_file)
        begin
          plugin_class = plugin[:class].constantize
          @plugin_list << plugin_class.new(plugin_environment)
        rescue => e
          log.error "Error while instantiating plugin: #{plugin[:name]}"
          log.exception e
        end
      end
    end


    def dry_run?
      ::Tomte.config.bender[:dry_run]
    end

    def banner
      log.info "== Service '#{self.class.name.underscore}'.".blue.bold
    end

    def verify_path(path, create = false, mask = nil, uid = nil, gid = nil)

      if(create)
        if(uid.nil? || gid.nil? || mask.nil?)
          raise(SystemCallError, "Asked to create a path with null mask/uid/gid")
        end
      end

      if(!FileTest::exists?(path))
        if(FileTest::directory?(path))
          return
        end

        if dry_run?
          log.info("DRYRUN should create: #{path}, mask=0#{mask.to_s(8)}, uid=#{uid}, gid=#{gid}")
        else
          FileUtils.mkdir_p(path)
          FileUtils.chmod(Integer(mask), path)
          FileUtils.chown(uid, gid, path)
        end

      elsif(!FileTest::directory?(path))
          raise(SystemCallError,"mandatory paths: file '#{path}' exists but is not a directory, skipping")
      end
    end

    private
    def base_path
      base_config[:base_path] || ''
    end

    def service_fail(reason)
      log.error reason.important!
      backend.shutdown!
    end

    def plugin_environment(hash = {})
      { :service => self,
        :host_identity => host_identity,
        :config => config,
        :base_config => base_config,
        :svc_config => svc_config,
        :templates_path => templates_path,
        :log => log,
        :dry_run => dry_run?,
      }.merge(hash)
    end

    def gem_templates_path
      "#{::Tomte.config.core[:application_lib]}/bender/templates/"
    end

    def exec_plugins(env, op = :provision)
      exec_rb_plugins(env, op)
      exec_native_plugins(env, op)
    end

    def exec_rb_plugins(env, op = :provision)
      plugin_list.each do |plugin|
        log.debug "+ Running plugin: #{plugin.class}"
        plugin.send(op, env)
      end
    end

    def exec_native_plugins(env, op = :provision)
      return unless Dir.exists?(native_plugins_path)
      list = Dir.entries(native_plugins_path).sort
      list.each do | plugin_file |
       next if(pluginfile.to_s =~ /^\./)
       native = File.join(native_plugins_path, plugin_file)
       log.debug("+ Plugins: executing native plugin '#{native}'")
       res = `#{native} #{mode}`
       log.info(res)
      end
    end
  end
end