module Bender
  module Services
    class DnsZones < Bender::ServiceImplementation

      MODEL_CLASS = Ygg::Dns::Zones

      def provision
        log.info "- provisioning for dns_zones"

      end

      def update(host = nil)

        if needs_update? || backend.config[:force]
          log.info "- updating dns zones"
          update_zones
          write_file File.join('agent', svc_config[:zones_config_master]), @master_list
          write_file File.join('agent', svc_config[:zones_config_slave]), @slave_list
          notify_server!
          register_update!
        else
          log.debug "- zones are up to date"
        end
      end

      def maintenance(host = nil)

        log.info "- doing mainenance "
      end


      def import_static(zone, file)
        text = File.read(file)
        z = MODEL_CLASS.find_by_name(zone)
        if z
          z.import_static zone, text
        else
          log.error "zone '#{zone}' not found"
        end
      rescue  Errno::ENOENT
        log.error "file '#{file}' not found"
      end


      def import_records(zone, file)
        text = File.read(file)
        z = MODEL_CLASS.find_by_name(zone)
        if z
          z.import_records zone, text
        else
          log.error "zone '#{zone}' not found"
        end
      rescue  Errno::ENOENT
        log.error "file '#{file}' not found"
      end

      def import_records_all
        all_zones.each do |z|
          if is_local_zone(z) && is_master_for?(z)
            master_file = master_file_path(z)

            if File.exist? master_file
              log.info "importing zone '#{z.name}' from file '#{master_file}'"
              import_records z.name, master_file
            end
          end
        end
      end

      def tasks
        puts "\nTasks:".yellow
        puts "import_static      ".blue + "<zone> <file>"
        puts "import_records     ".blue + "<zone> <file>"
        puts "import_records_all "
      end

      private

      def update_zones
        check_files_dir

        list_reset
        all_zones.each do |z|
          if is_local_zone(z)
            if is_master_for?(z)
              log.debug "+ this server is a master for zone #{z.name}"
              list_append_zone_master z
              configure_zone z
            else
              list_append_zone_slave z
              configure_slave z
            end
          end
        end
      end

      def is_master_for?(zone)
        zone.master_servers.any? {|s| s.server.node_name == local_node_name}
      end

      def list_reset
        @master_list = "//master zones\n// generated by bender, do not edit. Resource last update: #{resource_last_update}\n\n"
        @slave_list = "//slave zones\n// generated by bender, do not edit. Resource last update: #{resource_last_update}\n\n"
      end

      def list_append_zone_master(zone)
	      extra = ''
	      if File.exists?("/etc/named.d/extras/#{zone.name}.conf")
	        extra = "include \"/etc/named.d/extras/#{zone.name}.conf\";"
	      end
        @master_list += "zone \"#{zone.name}\" { type master; file \"#{master_zone_file(zone)}\"; #{extra}};\n"
      end

      def master_list_for(zone)
        l = zone.master_servers.inject([]) do |list, s|
          list << s.server.ipv4 if s.server.ipv4
          list << s.server.ipv6 if s.server.ipv6
          list
        end
        l.join("; ")
      end

      def list_append_zone_slave(zone)
        @slave_list += "zone \"#{zone.name}\" { type slave; file \"#{slave_zone_file(zone)}\"; masters { #{master_list_for(zone)}; }; };\n"
      end

      def config_files_dir
        File.join(base_path, svc_config[:zones_root])
      end

      def master_file_path(zone)
        File.join(config_files_dir, master_zone_file(zone))
      end

      def master_zone_file(zone)
        File.join('master', zone.zonefile)
      end

      def slave_zone_file(zone)
        File.join('slave', zone.zonefile)
      end

      def write_file(file, data)
        fileout = File.join(config_files_dir, file)
        verify_path(File.dirname(fileout), true, svc_config[:folders][:mode],
                    svc_config[:folders][:uid], svc_config[:folders][:gid])

        if dry_run?
          log.info "DRYRUN: should write '#{file}' to: '#{fileout}'."
        else
          File.open(fileout, "w+") do |f|
            f.write(data)
          end

          FileUtils.chmod(Integer(svc_config[:zones][:mode]), fileout)
          FileUtils.chown(svc_config[:zones][:uid], svc_config[:zones][:gid], fileout)
        end
      end

      def missing_zone_file_for?(zone)
        !File.file?(File.join(config_files_dir, master_zone_file(zone)))
      end

      def configure_zone(z)
        if missing_zone_file_for?(z)
          log.warn "+ zone file for #{z.name} is missing. Fetching it."
          write_file(master_zone_file(z), z.zone_file)
        else
          log.debug "+ zone file for #{z.name} has a zone file."
        end
      end

      def configure_slave(z)
        fileout = File.join(config_files_dir, slave_zone_file(z))
        verify_path(File.dirname(fileout), true, svc_config[:slave_folders][:mode],
                    svc_config[:slave_folders][:uid], svc_config[:slave_folders][:gid])

      end

      def check_files_dir
        [config_files_dir,
         File.join(config_files_dir, 'agent'),
         File.join(config_files_dir, 'master'),
         File.join(config_files_dir, 'slave')].each do |f|
          verify_path(f, true, svc_config[:folders][:mode], svc_config[:folders][:uid], svc_config[:folders][:gid])
        end

      end

      def is_local_zone(zone)
        zone.zone_servers.each do |s|
          if local_node_name ==  s.server.node_name
            log.debug "+ zone #{zone.name} is local on this server"
            return true
          end
        end
        log.debug "- zone #{zone.name} is not local"
        false
      end

      def local_node_name
        config[:local_node_name] || raise(ArgumentError, "- Local Node Name not configured")
      end

      def all_zones
        MODEL_CLASS.all( :params => { :view => :simple, :sort => :name, :scopes => :unprovisioned })
      end

      def resource_last_update
        @resource_last_update ||= MODEL_CLASS.last_update
      end

      def needs_update?
        tomte_debug do
          log.debug "Last update: #{last_update}, resource: #{resource_last_update}"
        end

        return true unless last_update
        last_update < resource_last_update
      end

      def register_update!
        self.last_update = resource_last_update
      end

      def notify_server!
        `rcnamed reload`
      end

    end
  end
end
